#!/usr/local/bin/perl
#
# getPhotoList.pl - Jim Bumgardner
#

# use JSON;

if (-e '/home/coverpop/perlmods/share/perl/5.8.8')
{
  use lib '/home/coverpop/perlmods/share/perl/5.8.8';
}

use Flickr::API;
use XML::Simple;
use Data::Dumper;
use Time::Local;

$Data::Dumper::Terse = 1;  # avoids $VAR1 = * ; in dumper output
$Data::Dumper::Indent = $verbose? 1 : 0;  # more concise output

# You will need to create this file...
# it supplies the authentication-related vars 
# $api_key and $sharedsecret
require 'apikey.ph';



$syntax = <<EOT;

  getPhotoList.pl [options] [<tags...>]
  getPhotoList.pl [options] -g group_id [<tag>]         
  getPhotoList.pl [options] -s photoset_id         
  getPhotoList.pl [options] -u username [<tags>]

Options:
  -all       Photos must match all tags (tag search only)
  -recent X  Only provide photos posted within the last X days (tag searches only)
  -limit X   Provide no more than X photos
  -license X Provide photos with license X
  -tags "x,y,z"
EOT

die $syntax if @ARGV == 0;

my $api = new Flickr::API({'key' => $api_key, secret => $sharedsecret});
$api->timeout(300);

$tags = '';
$ofname = '';
$method = 'flickr.photos.search';
$all = 0;
$extras = '';
$recent = 0;
$limit = 10000;
$license = 0;
$mintaken = 0;
$maxtaken = 0;
$getDescriptions = 0;
$verbose = 0;

while ($_ = shift)
{
  if (/^-interesting/)
  {
    $method = 'flickr.interestingness.getList';
    $ofname = 'flickr_interesting';
  }
  elsif (/^-g$/)
  {
    $group_id = shift;
    $method = 'flickr.groups.pools.getPhotos';
    print "Searching for photos in group $group_id\n";
    # determine output filename
    $ofname = $group_id;
  }
  elsif (/^-s$/) {
    $photoset_id = shift;
    $method = 'flickr.photosets.getPhotos' if !$tags;
    $ofname = $photoset_id;
  }
  elsif (/^-u$/)
  {
    $username = shift;
    $method = 'flickr.people.getPublicPhotos' if !$tags;
    $ofname = $username;
    $ofname =~ s/,\s*/_/g;
  }
  elsif (/^-all$/) {
    $all = 1;
  }
  elsif (/^-v(erbose)?$/) {
    $verbose = 1;
  }
  elsif (/^-mintaken$/) {
    $mintaken = shift;
    print "Parsing min $mintaken\n";
    my ($iyr,$imn,$imday,$ihour,$imin,$isec) = $mintaken =~ /(\d+)-(\d+)-(\d+) (\d+):(\d+):(\d+)/;
    $mintaken = timelocal($isec,$imin,$ihour,$imday,$imn-1,$iyr-1900);
  }
  elsif (/^-maxtaken$/) {
    $maxtaken = shift;
    print "Parsing max $maxtaken\n";
    my ($iyr,$imn,$imday,$ihour,$imin,$isec) = $maxtaken =~ /(\d+)-(\d+)-(\d+) (\d+):(\d+):(\d+)/;
    $maxtaken = timelocal($isec,$imin,$ihour,$imday,$imn-1,$iyr-1900);
  }
  elsif (/^-extras$/) {
    $extras = shift;
  }
  elsif (/^-limit$/) {
    $limit = shift;
  }
  elsif (/^-license$/) {
    $license = shift;
  }
  elsif (/^-recent$/) {
    $recent = shift;
  }
  elsif (/^-desc$/) {
    $getDescriptions = 1;
    $extras .= ',' if ($extras ne '');
    $extras .= 'last_update,machine_tags';
  }
  elsif (/^-/)
  {
    die $syntax;
  }
  else {
    $tags .= "," if $tags;
    $tags .= $_;
    print "Got tag $tags\n";
  }
}

if ($tags) {
  my $ftags = $tags;
  $ftags =~ s/,/_/g;
  $ofname .= '_' if $ofname;
  $ofname .= $ftags;
}

$ofname .= '.ph';
 
$nbrPages = 0;
$photoIdx = 0;


$user_id = '';
$min_taken_date = '';
$max_taken_date = '';

$xmlp = new XML::Simple ();

if ($recent) 
{
  die "-recent option only valid for tag search\n" if $method ne 'flickr.photos.search';
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
        gmtime(time - $recent*24*60*60);
  $min_taken_date = sprintf "%04d-%02d-%02d 00:00:00",1900+$year,$mon+1,$mday;
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
        gmtime(time);
  $max_taken_date = sprintf "%04d-%02d-%02d 00:00:00",1900+$year,$mon+1,$mday;
}

if ($username)
{
  # look up user id if a username was provided
  #
  if ($username =~ /\@/) {
    $user_id = $username;
  }
  else {
    my $response = $api->execute_method('flickr.people.findByUsername', {
                      username => $username} );
    die "Problem determining user_id: $response->{error_message}\n" if !$response->{success};

    print Dumper($response) if $verbose;  # explore results of call using -verbose

    my $xm = $xmlp->XMLin($response->decoded_content); 
    $user_id = $xm->{user}->{id};
  }
  print "Userid: $user_id\n";
}

print "License = $license\n" if $license;
print "Extras= $extras\n" if $extras;

%oldPhotos = ();

if (-e $ofname)
{
  require $ofname;
      # load last version of file here, to grab descriptions from...
  foreach my $photo (@books)
  {
    $oldPhotos{$photo->{id}} = $photo;
  }
  @books = ();
  $hasUpdates = 0;
}
else {
  $hasUpdates = 1;
}

open (OFILE, ">$ofname");
print OFILE "push \@books, (\n";

do
{
  my $params = {  per_page => 500,
                  page => $nbrPages+1,
                  auth_token => $auth_token};

  $params->{tags} = $tags if $tags;
  $params->{user_id} = $user_id if $user_id;
  $params->{group_id} = $group_id if $group_id;
  $params->{photoset_id} = $photoset_id if $photoset_id;
  $params->{min_taken_date} = $min_taken_date if $min_taken_date;
  $params->{max_taken_date} = $max_taken_date if $max_taken_date;
  $params->{license} = $license if $license;
  $params->{extras} = $extras if $extras;
  $params->{tag_mode} = 'all' if $all;

  print Dumper($params) . "\n" if $verbose;

  my $response = $api->execute_method($method, $params );
  die "Problem: $response->{error_message}\n" if !$response->{success};

  print Dumper($response) if $verbose;  # explore results of call using -verbose


  my $xml = $response->decoded_content;
#  $xml =~ s/ownername="[^"]+"//g;
#  $xml =~ s/title="[^"]+"//g;

  my $xm;

  printf "XML: %s\n", Dumper($xml) if $verbose;

  $xm = $xmlp->XMLin($xml,forcearray=>['photo']);
  $photos = $xm->{photos};
  $photos = $xm->{photoset} if $photoset_id;
#  $photos = $sm->{photoset}->{photo} if $photoset_id;

  # print OFILE Dumper($photos);

  printf "Page $photos->{page} of $photos->{pages} (%d photos, total=%d)\n", scalar(keys %{$photos->{photo}}), $photos->{total};
  printf "Parsed: %s\n", Dumper($photos) if $verbose;
  
  # loop thru photos
  $photoList = $photos->{photo};

  foreach $id (keys %{$photoList})
  {
    my $photo = $photoList->{$id};
    $photo->{id} = $id;
    $photo->{owner} = $photos->{owner} if !(defined $photo->{owner});
    if ($extras && $extras =~ /date_taken/)
    {
        # print "  ... $id ...\n";
        next if not defined $photo->{datetaken};
        next if $photo->{datetaken} eq '';
        my ($iyr,$imn,$imday,$ihour,$imin,$isec) = $photo->{datetaken} =~ /(\d+)-(\d+)-(\d+) (\d+):(\d+):(\d+)/;
        next if $iyr > 2018;
        $photo->{dt} = timelocal($isec,$imin,$ihour,$imday,$imn-1,$iyr-1900);
        next if $mintaken && ($photo->{dt} < $mintaken || $photo->{dt} > $maxtaken);
    }
    if ($getDescriptions) {
      if (defined $oldPhotos{$id} and $oldPhotos{$id}->{lastupdate} == $photo->{lastupdate})
      {
        # print "Loading existing desc\n";
        $photo->{desc} = $oldPhotos{$id}->{desc};
      }
      else {
        # print "Getting descriptions for id=$id\n";
        my $resp2 = $api->execute_method('flickr.photos.getInfo', {auth_token => $auth_token, photo_id => $id, secret => $photo->{secret}} );
        # printf "Info Str: %s\n", Dumper($resp2->decoded_content);
        # print "PROBLEM\n" if not defined $resp2->decoded_content;
        my $photoInfo = $xmlp->XMLin($resp2->decoded_content);
        # printf "Info: %s\n", Dumper($photoInfo);
        # printf "New Desc: %s\n", Dumper($photoInfo->{photo}->{description});
        $photo->{desc} = $photoInfo->{photo}->{description};
        $photo->{desc} =~ s~\\\'~'~g;
        $hasUpdates = 1;
      }
      if (defined $oldPhotos{$id} and $oldPhotos{$id}->{lastupdate} == $photo->{lastupdate} and defined  $oldPhotos{$id}->{city})
      {
        $photo->{city} = $oldPhotos{$id}->{city};
      }
      elsif (defined $photo->{machine_tags} and $photo->{machine_tags} =~ /apn:city=\w/i)
      {
        print "Getting machine tags for id=$id\n";
        my $resp2 = $api->execute_method('flickr.tags.getListPhoto', {auth_token => $auth_token, photo_id => $id} );
        print "PROBLEM\n" if not defined $resp2->decoded_content;
        my $photoInfo = $xmlp->XMLin($resp2->decoded_content);
        # print "GOT TAGS: %s\n", Dumper($photoInfo);
        my $tags = $photoInfo->{photo}->{tags}->{tag};
        foreach my $tkey (keys %$tags) {
          my $tag = $tags->{$tkey};
          if ($tag->{raw} =~ /apn:city=(.*)/) {
            $photo->{city} = $1;
            printf "GOT CITY: %s\n", $photo->{city};
            last;
          }
        }
      }
      else {
        $photo->{city} = '';
      }
    }
    else {
      $hasUpdates = 1 if not defined $oldPhotos{$id};
    }
    print OFILE ($photoIdx++? ",\n" : "") . Dumper($photo);
  }
  ++$nbrPages;
  
} while ($photos->{page} < $photos->{pages} && (!$limit || $photoIdx < $limit));

print OFILE "\n);\n\n\$hasUpdates = $hasUpdates;\n\n1;\n";
close OFILE;

print "$photoIdx photos written to $ofname\n";


